
'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/profiler/profiler_service_monitor_result.proto', package='tensorflow', syntax='proto3', serialized_options=None, create_key=_descriptor._internal_create_key, serialized_pb=b'\n>tensorflow/core/profiler/profiler_service_monitor_result.proto\x12\ntensorflow"\xad\x03\n\x1cProfilerServiceMonitorResult\x12L\n\rresponse_type\x18\x01 \x01(\x0e25.tensorflow.ProfilerServiceMonitorResult.ResponseType\x12 \n\x18device_idle_time_percent\x18\x02 \x01(\x01\x12\'\n\x1fmatrix_unit_utilization_percent\x18\x03 \x01(\x01\x12\x18\n\x10step_time_ms_avg\x18\x04 \x01(\x01\x12\x18\n\x10step_time_ms_min\x18\x05 \x01(\x01\x12\x18\n\x10step_time_ms_max\x18\x06 \x01(\x01\x12\x1a\n\x12infeed_percent_avg\x18\x07 \x01(\x01\x12\x1a\n\x12infeed_percent_min\x18\x08 \x01(\x01\x12\x1a\n\x12infeed_percent_max\x18\t \x01(\x01"R\n\x0cResponseType\x12\x10\n\x0cEMPTY_RESULT\x10\x00\x12\r\n\tUTIL_ONLY\x10\x01\x12\r\n\tUTIL_IDLE\x10\x02\x12\x12\n\x0eUTIL_IDLE_STEP\x10\x03b\x06proto3')
_PROFILERSERVICEMONITORRESULT_RESPONSETYPE = _descriptor.EnumDescriptor(name='ResponseType', full_name='tensorflow.ProfilerServiceMonitorResult.ResponseType', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='EMPTY_RESULT', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UTIL_ONLY', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UTIL_IDLE', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UTIL_IDLE_STEP', index=3, number=3, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=426, serialized_end=508)
_sym_db.RegisterEnumDescriptor(_PROFILERSERVICEMONITORRESULT_RESPONSETYPE)
_PROFILERSERVICEMONITORRESULT = _descriptor.Descriptor(name='ProfilerServiceMonitorResult', full_name='tensorflow.ProfilerServiceMonitorResult', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='response_type', full_name='tensorflow.ProfilerServiceMonitorResult.response_type', index=0, number=1, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='device_idle_time_percent', full_name='tensorflow.ProfilerServiceMonitorResult.device_idle_time_percent', index=1, number=2, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='matrix_unit_utilization_percent', full_name='tensorflow.ProfilerServiceMonitorResult.matrix_unit_utilization_percent', index=2, number=3, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='step_time_ms_avg', full_name='tensorflow.ProfilerServiceMonitorResult.step_time_ms_avg', index=3, number=4, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='step_time_ms_min', full_name='tensorflow.ProfilerServiceMonitorResult.step_time_ms_min', index=4, number=5, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='step_time_ms_max', full_name='tensorflow.ProfilerServiceMonitorResult.step_time_ms_max', index=5, number=6, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='infeed_percent_avg', full_name='tensorflow.ProfilerServiceMonitorResult.infeed_percent_avg', index=6, number=7, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='infeed_percent_min', full_name='tensorflow.ProfilerServiceMonitorResult.infeed_percent_min', index=7, number=8, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='infeed_percent_max', full_name='tensorflow.ProfilerServiceMonitorResult.infeed_percent_max', index=8, number=9, type=1, cpp_type=5, label=1, has_default_value=False, default_value=float(0), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[_PROFILERSERVICEMONITORRESULT_RESPONSETYPE], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=79, serialized_end=508)
_PROFILERSERVICEMONITORRESULT.fields_by_name['response_type'].enum_type = _PROFILERSERVICEMONITORRESULT_RESPONSETYPE
_PROFILERSERVICEMONITORRESULT_RESPONSETYPE.containing_type = _PROFILERSERVICEMONITORRESULT
DESCRIPTOR.message_types_by_name['ProfilerServiceMonitorResult'] = _PROFILERSERVICEMONITORRESULT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
ProfilerServiceMonitorResult = _reflection.GeneratedProtocolMessageType('ProfilerServiceMonitorResult', (_message.Message,), {'DESCRIPTOR': _PROFILERSERVICEMONITORRESULT, '__module__': 'tensorflow.core.profiler.profiler_service_monitor_result_pb2'})
_sym_db.RegisterMessage(ProfilerServiceMonitorResult)




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from ......tensorflow.compiler.xla.service import hlo_pb2 as tensorflow_dot_compiler_dot_xla_dot_service_dot_hlo__pb2
from ......tensorflow.compiler.xla import xla_data_pb2 as tensorflow_dot_compiler_dot_xla_dot_xla__data__pb2
from ......tensorflow.core.protobuf import autotuning_pb2 as tensorflow_dot_core_dot_protobuf_dot_autotuning__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/compiler/xla/service/gpu/gpu_autotuning.proto', package='xla.gpu', syntax='proto3', serialized_options=None, create_key=_descriptor._internal_create_key, serialized_pb=b'\n8tensorflow/compiler/xla/service/gpu/gpu_autotuning.proto\x12\x07xla.gpu\x1a)tensorflow/compiler/xla/service/hlo.proto\x1a&tensorflow/compiler/xla/xla_data.proto\x1a)tensorflow/core/protobuf/autotuning.proto"\x9f\x01\n\x12ConvInstructionLog\x12-\n\x0binstruction\x18\x01 \x01(\x0b2\x18.xla.HloInstructionProto\x12\'\n\x0eoperand_shapes\x18\x02 \x03(\x0b2\x0f.xla.ShapeProto\x12\x16\n\x0eresult_address\x18\x03 \x01(\x04\x12\x19\n\x11operand_addresses\x18\x04 \x03(\x04"5\n\x13DenylistedAlgorithm\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x12\n\ntensor_ops\x18\x02 \x01(\x08"\xc4\x01\n\x16AlgorithmDenylistEntry\x12\x0b\n\x03hlo\x18\x01 \x01(\t\x12)\n\x02cc\x18\x02 \x01(\x0b2\x1d.tensorflow.ComputeCapability\x12/\n\rcudnn_version\x18\x03 \x01(\x0b2\x18.tensorflow.CudnnVersion\x12\x14\n\x0cblas_version\x18\x05 \x01(\t\x12+\n\x05algos\x18\x04 \x03(\x0b2\x1c.xla.gpu.DenylistedAlgorithm"E\n\x11AlgorithmDenylist\x120\n\x07entries\x18\x01 \x03(\x0b2\x1f.xla.gpu.AlgorithmDenylistEntryb\x06proto3', dependencies=[tensorflow_dot_compiler_dot_xla_dot_service_dot_hlo__pb2.DESCRIPTOR, tensorflow_dot_compiler_dot_xla_dot_xla__data__pb2.DESCRIPTOR, tensorflow_dot_core_dot_protobuf_dot_autotuning__pb2.DESCRIPTOR])
_CONVINSTRUCTIONLOG = _descriptor.Descriptor(name='ConvInstructionLog', full_name='xla.gpu.ConvInstructionLog', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='instruction', full_name='xla.gpu.ConvInstructionLog.instruction', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='operand_shapes', full_name='xla.gpu.ConvInstructionLog.operand_shapes', index=1, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='result_address', full_name='xla.gpu.ConvInstructionLog.result_address', index=2, number=3, type=4, cpp_type=4, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='operand_addresses', full_name='xla.gpu.ConvInstructionLog.operand_addresses', index=3, number=4, type=4, cpp_type=4, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=196, serialized_end=355)
_DENYLISTEDALGORITHM = _descriptor.Descriptor(name='DenylistedAlgorithm', full_name='xla.gpu.DenylistedAlgorithm', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='id', full_name='xla.gpu.DenylistedAlgorithm.id', index=0, number=1, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='tensor_ops', full_name='xla.gpu.DenylistedAlgorithm.tensor_ops', index=1, number=2, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=357, serialized_end=410)
_ALGORITHMDENYLISTENTRY = _descriptor.Descriptor(name='AlgorithmDenylistEntry', full_name='xla.gpu.AlgorithmDenylistEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='hlo', full_name='xla.gpu.AlgorithmDenylistEntry.hlo', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='cc', full_name='xla.gpu.AlgorithmDenylistEntry.cc', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='cudnn_version', full_name='xla.gpu.AlgorithmDenylistEntry.cudnn_version', index=2, number=3, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='blas_version', full_name='xla.gpu.AlgorithmDenylistEntry.blas_version', index=3, number=5, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='algos', full_name='xla.gpu.AlgorithmDenylistEntry.algos', index=4, number=4, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=413, serialized_end=609)
_ALGORITHMDENYLIST = _descriptor.Descriptor(name='AlgorithmDenylist', full_name='xla.gpu.AlgorithmDenylist', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='entries', full_name='xla.gpu.AlgorithmDenylist.entries', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=611, serialized_end=680)
_CONVINSTRUCTIONLOG.fields_by_name['instruction'].message_type = tensorflow_dot_compiler_dot_xla_dot_service_dot_hlo__pb2._HLOINSTRUCTIONPROTO
_CONVINSTRUCTIONLOG.fields_by_name['operand_shapes'].message_type = tensorflow_dot_compiler_dot_xla_dot_xla__data__pb2._SHAPEPROTO
_ALGORITHMDENYLISTENTRY.fields_by_name['cc'].message_type = tensorflow_dot_core_dot_protobuf_dot_autotuning__pb2._COMPUTECAPABILITY
_ALGORITHMDENYLISTENTRY.fields_by_name['cudnn_version'].message_type = tensorflow_dot_core_dot_protobuf_dot_autotuning__pb2._CUDNNVERSION
_ALGORITHMDENYLISTENTRY.fields_by_name['algos'].message_type = _DENYLISTEDALGORITHM
_ALGORITHMDENYLIST.fields_by_name['entries'].message_type = _ALGORITHMDENYLISTENTRY
DESCRIPTOR.message_types_by_name['ConvInstructionLog'] = _CONVINSTRUCTIONLOG
DESCRIPTOR.message_types_by_name['DenylistedAlgorithm'] = _DENYLISTEDALGORITHM
DESCRIPTOR.message_types_by_name['AlgorithmDenylistEntry'] = _ALGORITHMDENYLISTENTRY
DESCRIPTOR.message_types_by_name['AlgorithmDenylist'] = _ALGORITHMDENYLIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
ConvInstructionLog = _reflection.GeneratedProtocolMessageType('ConvInstructionLog', (_message.Message,), {'DESCRIPTOR': _CONVINSTRUCTIONLOG, '__module__': 'tensorflow.compiler.xla.service.gpu.gpu_autotuning_pb2'})
_sym_db.RegisterMessage(ConvInstructionLog)
DenylistedAlgorithm = _reflection.GeneratedProtocolMessageType('DenylistedAlgorithm', (_message.Message,), {'DESCRIPTOR': _DENYLISTEDALGORITHM, '__module__': 'tensorflow.compiler.xla.service.gpu.gpu_autotuning_pb2'})
_sym_db.RegisterMessage(DenylistedAlgorithm)
AlgorithmDenylistEntry = _reflection.GeneratedProtocolMessageType('AlgorithmDenylistEntry', (_message.Message,), {'DESCRIPTOR': _ALGORITHMDENYLISTENTRY, '__module__': 'tensorflow.compiler.xla.service.gpu.gpu_autotuning_pb2'})
_sym_db.RegisterMessage(AlgorithmDenylistEntry)
AlgorithmDenylist = _reflection.GeneratedProtocolMessageType('AlgorithmDenylist', (_message.Message,), {'DESCRIPTOR': _ALGORITHMDENYLIST, '__module__': 'tensorflow.compiler.xla.service.gpu.gpu_autotuning_pb2'})
_sym_db.RegisterMessage(AlgorithmDenylist)




'Generated protocol buffer code.'
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/lite/toco/types.proto', package='toco', syntax='proto2', serialized_options=None, create_key=_descriptor._internal_create_key, serialized_pb=b'\n tensorflow/lite/toco/types.proto\x12\x04toco*\xa7\x02\n\nIODataType\x12\x18\n\x14IO_DATA_TYPE_UNKNOWN\x10\x00\x12\t\n\x05FLOAT\x10\x01\x12\x13\n\x0fQUANTIZED_UINT8\x10\x02\x12\t\n\x05INT32\x10\x03\x12\t\n\x05INT64\x10\x04\x12\n\n\x06STRING\x10\x05\x12\x13\n\x0fQUANTIZED_INT16\x10\x06\x12\x08\n\x04BOOL\x10\x07\x12\r\n\tCOMPLEX64\x10\x08\x12\x12\n\x0eQUANTIZED_INT8\x10\t\x12\x0b\n\x07FLOAT16\x10\n\x12\x0b\n\x07FLOAT64\x10\x0b\x12\x0e\n\nCOMPLEX128\x10\x0c\x12\n\n\x06UINT64\x10\r\x12\x0c\n\x08RESOURCE\x10\x0e\x12\x0b\n\x07VARIANT\x10\x0f\x12\n\n\x06UINT32\x10\x10\x12\t\n\x05UINT8\x10\x11\x12\x08\n\x04INT8\x10\x12\x12\t\n\x05INT16\x10\x13')
_IODATATYPE = _descriptor.EnumDescriptor(name='IODataType', full_name='toco.IODataType', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='IO_DATA_TYPE_UNKNOWN', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='FLOAT', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='QUANTIZED_UINT8', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='INT32', index=3, number=3, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='INT64', index=4, number=4, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='STRING', index=5, number=5, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='QUANTIZED_INT16', index=6, number=6, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='BOOL', index=7, number=7, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='COMPLEX64', index=8, number=8, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='QUANTIZED_INT8', index=9, number=9, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='FLOAT16', index=10, number=10, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='FLOAT64', index=11, number=11, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='COMPLEX128', index=12, number=12, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UINT64', index=13, number=13, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='RESOURCE', index=14, number=14, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='VARIANT', index=15, number=15, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UINT32', index=16, number=16, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='UINT8', index=17, number=17, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='INT8', index=18, number=18, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='INT16', index=19, number=19, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=43, serialized_end=338)
_sym_db.RegisterEnumDescriptor(_IODATATYPE)
IODataType = enum_type_wrapper.EnumTypeWrapper(_IODATATYPE)
IO_DATA_TYPE_UNKNOWN = 0
FLOAT = 1
QUANTIZED_UINT8 = 2
INT32 = 3
INT64 = 4
STRING = 5
QUANTIZED_INT16 = 6
BOOL = 7
COMPLEX64 = 8
QUANTIZED_INT8 = 9
FLOAT16 = 10
FLOAT64 = 11
COMPLEX128 = 12
UINT64 = 13
RESOURCE = 14
VARIANT = 15
UINT32 = 16
UINT8 = 17
INT8 = 18
INT16 = 19
DESCRIPTOR.enum_types_by_name['IODataType'] = _IODATATYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from ....tensorflow.core.framework import attr_value_pb2 as tensorflow_dot_core_dot_framework_dot_attr__value__pb2
from ....tensorflow.core.protobuf import verifier_config_pb2 as tensorflow_dot_core_dot_protobuf_dot_verifier__config__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/protobuf/rewriter_config.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x18org.tensorflow.frameworkB\x14RewriterConfigProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n.tensorflow/core/protobuf/rewriter_config.proto\x12\ntensorflow\x1a*tensorflow/core/framework/attr_value.proto\x1a.tensorflow/core/protobuf/verifier_config.proto";\n\x13AutoParallelOptions\x12\x0e\n\x06enable\x18\x01 \x01(\x08\x12\x14\n\x0cnum_replicas\x18\x02 \x01(\x05"+\n\x16ScopedAllocatorOptions\x12\x11\n\tenable_op\x18\x01 \x03(\t"\xe1\x13\n\x0eRewriterConfig\x12C\n\x15cpu_layout_conversion\x182 \x01(\x0e2$.tensorflow.RewriterConfig.CpuLayout\x12;\n\x10layout_optimizer\x18\x01 \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12;\n\x10constant_folding\x18\x03 \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12=\n\x12shape_optimization\x18\r \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x124\n\tremapping\x18\x0e \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12F\n\x1bcommon_subgraph_elimination\x18\x18 \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12B\n\x17arithmetic_optimization\x18\x07 \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12B\n\x17dependency_optimization\x18\x08 \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12<\n\x11loop_optimization\x18\t \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12@\n\x15function_optimization\x18\n \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x129\n\x0edebug_stripper\x18\x0b \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12\x1d\n\x15disable_model_pruning\x18\x02 \x01(\x08\x12H\n\x1dscoped_allocator_optimization\x18\x0f \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12C\n\x18pin_to_host_optimization\x18\x12 \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12B\n\x17implementation_selector\x18\x16 \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12?\n\x14auto_mixed_precision\x18\x17 \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12C\n\x18auto_mixed_precision_mkl\x18\x19 \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12\x1e\n\x16disable_meta_optimizer\x18\x13 \x01(\x08\x12@\n\x15use_plugin_optimizers\x18\x1c \x01(\x0e2!.tensorflow.RewriterConfig.Toggle\x12O\n\x19meta_optimizer_iterations\x18\x0c \x01(\x0e2,.tensorflow.RewriterConfig.NumIterationsType\x12\x17\n\x0fmin_graph_nodes\x18\x11 \x01(\x05\x12;\n3experimental_disable_compressed_tensor_optimization\x18\x1a \x01(\x08\x12;\n3experimental_disable_folding_quantization_emulation\x18\x1b \x01(\x08\x12B\n\x13memory_optimization\x18\x04 \x01(\x0e2%.tensorflow.RewriterConfig.MemOptType\x12/\n\'memory_optimizer_target_node_name_scope\x18\x06 \x01(\t\x12!\n\x19meta_optimizer_timeout_ms\x18\x14 \x01(\x03\x126\n\rauto_parallel\x18\x05 \x01(\x0b2\x1f.tensorflow.AutoParallelOptions\x12 \n\x18fail_on_optimizer_errors\x18\x15 \x01(\x08\x12A\n\x15scoped_allocator_opts\x18\x10 \x01(\x0b2".tensorflow.ScopedAllocatorOptions\x12\x12\n\noptimizers\x18d \x03(\t\x12K\n\x11custom_optimizers\x18\xc8\x01 \x03(\x0b2/.tensorflow.RewriterConfig.CustomGraphOptimizer\x12D\n\x1finter_optimizer_verifier_config\x18\xac\x02 \x01(\x0b2\x1a.tensorflow.VerifierConfig\x12F\n!post_optimization_verifier_config\x18\xad\x02 \x01(\x0b2\x1a.tensorflow.VerifierConfig\x1a\xca\x01\n\x14CustomGraphOptimizer\x12\x0c\n\x04name\x18\x01 \x01(\t\x12X\n\rparameter_map\x18\x02 \x03(\x0b2A.tensorflow.RewriterConfig.CustomGraphOptimizer.ParameterMapEntry\x1aJ\n\x11ParameterMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b2\x15.tensorflow.AttrValue:\x028\x01"6\n\x06Toggle\x12\x0b\n\x07DEFAULT\x10\x00\x12\x06\n\x02ON\x10\x01\x12\x07\n\x03OFF\x10\x02\x12\x0e\n\nAGGRESSIVE\x10\x03"I\n\tCpuLayout\x12\x18\n\x14NO_CONVERSION_ON_CPU\x10\x00\x12\x10\n\x0cNCHW_TO_NHWC\x10\x01\x12\x10\n\x0cNHWC_TO_NCHW\x10\x02"<\n\x11NumIterationsType\x12\x15\n\x11DEFAULT_NUM_ITERS\x10\x00\x12\x07\n\x03ONE\x10\x01\x12\x07\n\x03TWO\x10\x02"\x9f\x01\n\nMemOptType\x12\x13\n\x0fDEFAULT_MEM_OPT\x10\x00\x12\x0e\n\nNO_MEM_OPT\x10\x01\x12\n\n\x06MANUAL\x10\x02\x12\x17\n\x13SWAPPING_HEURISTICS\x10\x04\x12\x1c\n\x18RECOMPUTATION_HEURISTICS\x10\x05\x12\x19\n\x15SCHEDULING_HEURISTICS\x10\x06\x12\x0e\n\nHEURISTICS\x10\x03B\x8c\x01\n\x18org.tensorflow.frameworkB\x14RewriterConfigProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01b\x06proto3', dependencies=[tensorflow_dot_core_dot_framework_dot_attr__value__pb2.DESCRIPTOR, tensorflow_dot_core_dot_protobuf_dot_verifier__config__pb2.DESCRIPTOR])
_REWRITERCONFIG_TOGGLE = _descriptor.EnumDescriptor(name='Toggle', full_name='tensorflow.RewriterConfig.Toggle', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='DEFAULT', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='ON', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='OFF', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='AGGRESSIVE', index=3, number=3, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=2437, serialized_end=2491)
_sym_db.RegisterEnumDescriptor(_REWRITERCONFIG_TOGGLE)
_REWRITERCONFIG_CPULAYOUT = _descriptor.EnumDescriptor(name='CpuLayout', full_name='tensorflow.RewriterConfig.CpuLayout', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='NO_CONVERSION_ON_CPU', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='NCHW_TO_NHWC', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='NHWC_TO_NCHW', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=2493, serialized_end=2566)
_sym_db.RegisterEnumDescriptor(_REWRITERCONFIG_CPULAYOUT)
_REWRITERCONFIG_NUMITERATIONSTYPE = _descriptor.EnumDescriptor(name='NumIterationsType', full_name='tensorflow.RewriterConfig.NumIterationsType', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='DEFAULT_NUM_ITERS', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='ONE', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='TWO', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=2568, serialized_end=2628)
_sym_db.RegisterEnumDescriptor(_REWRITERCONFIG_NUMITERATIONSTYPE)
_REWRITERCONFIG_MEMOPTTYPE = _descriptor.EnumDescriptor(name='MemOptType', full_name='tensorflow.RewriterConfig.MemOptType', filename=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key, values=[_descriptor.EnumValueDescriptor(name='DEFAULT_MEM_OPT', index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='NO_MEM_OPT', index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='MANUAL', index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='SWAPPING_HEURISTICS', index=3, number=4, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='RECOMPUTATION_HEURISTICS', index=4, number=5, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='SCHEDULING_HEURISTICS', index=5, number=6, serialized_options=None, type=None, create_key=_descriptor._internal_create_key), _descriptor.EnumValueDescriptor(name='HEURISTICS', index=6, number=3, serialized_options=None, type=None, create_key=_descriptor._internal_create_key)], containing_type=None, serialized_options=None, serialized_start=2631, serialized_end=2790)
_sym_db.RegisterEnumDescriptor(_REWRITERCONFIG_MEMOPTTYPE)
_AUTOPARALLELOPTIONS = _descriptor.Descriptor(name='AutoParallelOptions', full_name='tensorflow.AutoParallelOptions', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='enable', full_name='tensorflow.AutoParallelOptions.enable', index=0, number=1, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='num_replicas', full_name='tensorflow.AutoParallelOptions.num_replicas', index=1, number=2, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=154, serialized_end=213)
_SCOPEDALLOCATOROPTIONS = _descriptor.Descriptor(name='ScopedAllocatorOptions', full_name='tensorflow.ScopedAllocatorOptions', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='enable_op', full_name='tensorflow.ScopedAllocatorOptions.enable_op', index=0, number=1, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=215, serialized_end=258)
_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY = _descriptor.Descriptor(name='ParameterMapEntry', full_name='tensorflow.RewriterConfig.CustomGraphOptimizer.ParameterMapEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.RewriterConfig.CustomGraphOptimizer.ParameterMapEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.RewriterConfig.CustomGraphOptimizer.ParameterMapEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=2361, serialized_end=2435)
_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER = _descriptor.Descriptor(name='CustomGraphOptimizer', full_name='tensorflow.RewriterConfig.CustomGraphOptimizer', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='name', full_name='tensorflow.RewriterConfig.CustomGraphOptimizer.name', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='parameter_map', full_name='tensorflow.RewriterConfig.CustomGraphOptimizer.parameter_map', index=1, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=2233, serialized_end=2435)
_REWRITERCONFIG = _descriptor.Descriptor(name='RewriterConfig', full_name='tensorflow.RewriterConfig', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='cpu_layout_conversion', full_name='tensorflow.RewriterConfig.cpu_layout_conversion', index=0, number=50, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='layout_optimizer', full_name='tensorflow.RewriterConfig.layout_optimizer', index=1, number=1, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='constant_folding', full_name='tensorflow.RewriterConfig.constant_folding', index=2, number=3, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='shape_optimization', full_name='tensorflow.RewriterConfig.shape_optimization', index=3, number=13, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='remapping', full_name='tensorflow.RewriterConfig.remapping', index=4, number=14, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='common_subgraph_elimination', full_name='tensorflow.RewriterConfig.common_subgraph_elimination', index=5, number=24, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='arithmetic_optimization', full_name='tensorflow.RewriterConfig.arithmetic_optimization', index=6, number=7, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='dependency_optimization', full_name='tensorflow.RewriterConfig.dependency_optimization', index=7, number=8, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='loop_optimization', full_name='tensorflow.RewriterConfig.loop_optimization', index=8, number=9, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='function_optimization', full_name='tensorflow.RewriterConfig.function_optimization', index=9, number=10, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='debug_stripper', full_name='tensorflow.RewriterConfig.debug_stripper', index=10, number=11, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='disable_model_pruning', full_name='tensorflow.RewriterConfig.disable_model_pruning', index=11, number=2, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='scoped_allocator_optimization', full_name='tensorflow.RewriterConfig.scoped_allocator_optimization', index=12, number=15, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='pin_to_host_optimization', full_name='tensorflow.RewriterConfig.pin_to_host_optimization', index=13, number=18, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='implementation_selector', full_name='tensorflow.RewriterConfig.implementation_selector', index=14, number=22, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='auto_mixed_precision', full_name='tensorflow.RewriterConfig.auto_mixed_precision', index=15, number=23, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='auto_mixed_precision_mkl', full_name='tensorflow.RewriterConfig.auto_mixed_precision_mkl', index=16, number=25, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='disable_meta_optimizer', full_name='tensorflow.RewriterConfig.disable_meta_optimizer', index=17, number=19, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='use_plugin_optimizers', full_name='tensorflow.RewriterConfig.use_plugin_optimizers', index=18, number=28, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='meta_optimizer_iterations', full_name='tensorflow.RewriterConfig.meta_optimizer_iterations', index=19, number=12, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='min_graph_nodes', full_name='tensorflow.RewriterConfig.min_graph_nodes', index=20, number=17, type=5, cpp_type=1, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='experimental_disable_compressed_tensor_optimization', full_name='tensorflow.RewriterConfig.experimental_disable_compressed_tensor_optimization', index=21, number=26, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='experimental_disable_folding_quantization_emulation', full_name='tensorflow.RewriterConfig.experimental_disable_folding_quantization_emulation', index=22, number=27, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='memory_optimization', full_name='tensorflow.RewriterConfig.memory_optimization', index=23, number=4, type=14, cpp_type=8, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='memory_optimizer_target_node_name_scope', full_name='tensorflow.RewriterConfig.memory_optimizer_target_node_name_scope', index=24, number=6, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='meta_optimizer_timeout_ms', full_name='tensorflow.RewriterConfig.meta_optimizer_timeout_ms', index=25, number=20, type=3, cpp_type=2, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='auto_parallel', full_name='tensorflow.RewriterConfig.auto_parallel', index=26, number=5, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='fail_on_optimizer_errors', full_name='tensorflow.RewriterConfig.fail_on_optimizer_errors', index=27, number=21, type=8, cpp_type=7, label=1, has_default_value=False, default_value=False, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='scoped_allocator_opts', full_name='tensorflow.RewriterConfig.scoped_allocator_opts', index=28, number=16, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='optimizers', full_name='tensorflow.RewriterConfig.optimizers', index=29, number=100, type=9, cpp_type=9, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='custom_optimizers', full_name='tensorflow.RewriterConfig.custom_optimizers', index=30, number=200, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='inter_optimizer_verifier_config', full_name='tensorflow.RewriterConfig.inter_optimizer_verifier_config', index=31, number=300, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='post_optimization_verifier_config', full_name='tensorflow.RewriterConfig.post_optimization_verifier_config', index=32, number=301, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER], enum_types=[_REWRITERCONFIG_TOGGLE, _REWRITERCONFIG_CPULAYOUT, _REWRITERCONFIG_NUMITERATIONSTYPE, _REWRITERCONFIG_MEMOPTTYPE], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=261, serialized_end=2790)
_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY.fields_by_name['value'].message_type = tensorflow_dot_core_dot_framework_dot_attr__value__pb2._ATTRVALUE
_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY.containing_type = _REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER
_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER.fields_by_name['parameter_map'].message_type = _REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY
_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER.containing_type = _REWRITERCONFIG
_REWRITERCONFIG.fields_by_name['cpu_layout_conversion'].enum_type = _REWRITERCONFIG_CPULAYOUT
_REWRITERCONFIG.fields_by_name['layout_optimizer'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['constant_folding'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['shape_optimization'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['remapping'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['common_subgraph_elimination'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['arithmetic_optimization'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['dependency_optimization'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['loop_optimization'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['function_optimization'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['debug_stripper'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['scoped_allocator_optimization'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['pin_to_host_optimization'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['implementation_selector'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['auto_mixed_precision'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['auto_mixed_precision_mkl'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['use_plugin_optimizers'].enum_type = _REWRITERCONFIG_TOGGLE
_REWRITERCONFIG.fields_by_name['meta_optimizer_iterations'].enum_type = _REWRITERCONFIG_NUMITERATIONSTYPE
_REWRITERCONFIG.fields_by_name['memory_optimization'].enum_type = _REWRITERCONFIG_MEMOPTTYPE
_REWRITERCONFIG.fields_by_name['auto_parallel'].message_type = _AUTOPARALLELOPTIONS
_REWRITERCONFIG.fields_by_name['scoped_allocator_opts'].message_type = _SCOPEDALLOCATOROPTIONS
_REWRITERCONFIG.fields_by_name['custom_optimizers'].message_type = _REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER
_REWRITERCONFIG.fields_by_name['inter_optimizer_verifier_config'].message_type = tensorflow_dot_core_dot_protobuf_dot_verifier__config__pb2._VERIFIERCONFIG
_REWRITERCONFIG.fields_by_name['post_optimization_verifier_config'].message_type = tensorflow_dot_core_dot_protobuf_dot_verifier__config__pb2._VERIFIERCONFIG
_REWRITERCONFIG_TOGGLE.containing_type = _REWRITERCONFIG
_REWRITERCONFIG_CPULAYOUT.containing_type = _REWRITERCONFIG
_REWRITERCONFIG_NUMITERATIONSTYPE.containing_type = _REWRITERCONFIG
_REWRITERCONFIG_MEMOPTTYPE.containing_type = _REWRITERCONFIG
DESCRIPTOR.message_types_by_name['AutoParallelOptions'] = _AUTOPARALLELOPTIONS
DESCRIPTOR.message_types_by_name['ScopedAllocatorOptions'] = _SCOPEDALLOCATOROPTIONS
DESCRIPTOR.message_types_by_name['RewriterConfig'] = _REWRITERCONFIG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
AutoParallelOptions = _reflection.GeneratedProtocolMessageType('AutoParallelOptions', (_message.Message,), {'DESCRIPTOR': _AUTOPARALLELOPTIONS, '__module__': 'tensorflow.core.protobuf.rewriter_config_pb2'})
_sym_db.RegisterMessage(AutoParallelOptions)
ScopedAllocatorOptions = _reflection.GeneratedProtocolMessageType('ScopedAllocatorOptions', (_message.Message,), {'DESCRIPTOR': _SCOPEDALLOCATOROPTIONS, '__module__': 'tensorflow.core.protobuf.rewriter_config_pb2'})
_sym_db.RegisterMessage(ScopedAllocatorOptions)
RewriterConfig = _reflection.GeneratedProtocolMessageType('RewriterConfig', (_message.Message,), {'CustomGraphOptimizer': _reflection.GeneratedProtocolMessageType('CustomGraphOptimizer', (_message.Message,), {'ParameterMapEntry': _reflection.GeneratedProtocolMessageType('ParameterMapEntry', (_message.Message,), {'DESCRIPTOR': _REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY, '__module__': 'tensorflow.core.protobuf.rewriter_config_pb2'}), 'DESCRIPTOR': _REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER, '__module__': 'tensorflow.core.protobuf.rewriter_config_pb2'}), 'DESCRIPTOR': _REWRITERCONFIG, '__module__': 'tensorflow.core.protobuf.rewriter_config_pb2'})
_sym_db.RegisterMessage(RewriterConfig)
_sym_db.RegisterMessage(RewriterConfig.CustomGraphOptimizer)
_sym_db.RegisterMessage(RewriterConfig.CustomGraphOptimizer.ParameterMapEntry)
DESCRIPTOR._options = None
_REWRITERCONFIG_CUSTOMGRAPHOPTIMIZER_PARAMETERMAPENTRY._options = None



from enum import Enum, auto
from typing import Union

from . import core
from . import str_util

# from fifostr import FIFOStr
#
# class string(FIFOStr):
#   def __init__(self, value = ""):
#     super().__init__(2 ** 63 - 1)
#     self.append(value)
#
#   def set(self, value: str):
#     self.clear()
#     self.append(str(value))

def resize(s: bytearray, size: int):
  assert size >= 0
  if len(s) > size:
    s[:] = s[:size]
  if len(s) < size:
    s.extend(bytearray(size - len(s)))

def str_cat(*args):
  s = []
  for arg in args:
    if isinstance(arg, str):
      s.append(arg)
    else:
      arg = bytes(arg)
      s.append(arg.decode('utf-8'))
  return ''.join(s)

def printf(format, *args) -> bytes:
  format = core.string_view(format).bytes()
  format = format.replace(b"%llx", b"%x")
  return format % args

class Ascii:
  LOWER_A = ord('a')
  LOWER_Z = ord('z')
  UPPER_A = ord('A')
  UPPER_Z = ord('Z')
  DIGIT_0 = ord('0')
  DIGIT_9 = ord('9')
  DOT = ord('.')
  DASH = ord('-')
  UNDERSCORE = ord('_')
  SLASH = ord('/')
  PLUS = ord('+')
  MINUS = ord('-')
  RANGLE = ord('>')

class Scanner:
  class CharClass(Enum):
    # NOTE: When adding a new CharClass, update the AllCharClasses ScannerTest
    # in scanner_test.cc
    ALL = auto()
    DIGIT = auto()
    LETTER = auto()
    LETTER_DIGIT = auto()
    LETTER_DIGIT_DASH_UNDERSCORE = auto()
    LETTER_DIGIT_DASH_DOT_SLASH = auto()             # SLASH is / only, not backslash
    LETTER_DIGIT_DASH_DOT_SLASH_UNDERSCORE = auto()  # SLASH is / only, not backslash
    LETTER_DIGIT_DOT = auto()
    LETTER_DIGIT_DOT_PLUS_MINUS = auto()
    LETTER_DIGIT_DOT_UNDERSCORE = auto()
    LETTER_DIGIT_UNDERSCORE = auto()
    LOWERLETTER = auto()
    LOWERLETTER_DIGIT = auto()
    LOWERLETTER_DIGIT_UNDERSCORE = auto()
    NON_ZERO_DIGIT = auto()
    SPACE = auto()
    UPPERLETTER = auto()
    RANGLE = auto()

  def __init__(self, source: core.StringPiece):
    self._error = False
    self._capture_start = None
    self._capture_end = None
    self._cur = core.string_view(source)
    self.restart_capture()

  def one(self, clz: CharClass):
    """Consume the next character of the given class from input. If the next
    character is not in the class, then GetResult will ultimately return false."""
    if self._cur.empty() or not self.matches(clz, self._cur[0]):
      return self.error()
    self._cur.remove_prefix(1)
    return self

  def zero_or_one_literal(self, s: core.StringPiece):
    """Consume the next s.size() characters of the input, if they match <s>. If
    they don't match <s>, this is a no-op."""
    str_util.consume_prefix(self._cur, s)
    return self

  def one_literal(self, s: core.StringPiece):
    """Consume the next s.size() characters of the input, if they match <s>. If
    they don't match <s>, then GetResult will ultimately return false."""
    if not str_util.consume_prefix(self._cur, s):
      self._error = True
    return self

  def any(self, clz: CharClass):
    """Consume characters from the input as long as they match <clz>. Zero
    characters is still considered a match, so it will never cause GetResult to
    return false."""
    while not self._cur.empty() and self.matches(clz, self._cur[0]):
      self._cur.remove_prefix(1)
    return self

  def many(self, clz: CharClass):
    """Shorthand for One(clz).Any(clz)."""
    return self.one(clz).any(clz)

  def restart_capture(self):
    """Reset the capture start point.

    Later, when GetResult is called and if it returns true, the capture
    returned will start at the position at the time this was called."""
    self._capture_start = self._cur.begin()
    self._capture_end = None
    return self

  def stop_capture(self):
    """Stop capturing input.

    Later, when GetResult is called and if it returns true, the capture
    returned will end at the position at the time this was called."""
    self._capture_end = self._cur.begin()
    return self

  def eos(self):
    """If not at the input of input, then get_result will ultimately return false."""
    if not self._cur.empty():
      self._error = True
    return self

  def any_space(self):
    """Shorthand for Any(SPACE)."""
    return self.any(Scanner.CharClass.SPACE)

  def scan_until(self, end_ch):
    """This scans input until <end_ch> is reached. <end_ch> is NOT consumed."""
    self.scan_until_impl(end_ch, escaped=False)
    return self

  def scan_escaped_until(self, end_ch):
    """This scans input until <end_ch> is reached. <end_ch> is NOT consumed.
    Backslash escape sequences are skipped.
    Used for implementing quoted string scanning."""
    self.scan_until_impl(end_ch, escaped=True)
    return self

  def peek(self,default_value = 0):
    """Return the next character that will be scanned, or <default_value> if there
    are no more characters to scan.
    Note that if a scan operation has failed (so GetResult() returns false),
    then the value of Peek may or may not have advanced since the scan
    operation that failed."""
    return default_value if self._cur.empty() else self._cur[0]

  def empty(self) -> bool:
    """Returns false if there are no remaining characters to consume."""
    return self._cur.empty()

  def scan_until_impl(self, end_ch, escaped: bool):
    while True:
      if self._cur.empty():
        self.error()
        return
      ch = self._cur[0]
      if ch == end_ch:
        return

      self._cur.remove_prefix(1)
      if escaped and ch == 92:  # backslash
        # Escape character, skip next character.
        if self._cur.empty():
          self.error()
          return
        self._cur.remove_prefix(1)

  def get_result(self, remaining: core.StringPiece = None, capture: core.StringPiece = None) -> bool:
    """Returns true if the input string successfully matched. When true is
    returned, the remaining string is returned in <remaining> and the captured
    string returned in <capture>, if non-NULL."""
    if self._error:
      return False
    if remaining is not None:
      remaining.set(self._cur)
    if capture is not None:
      end = self._cur.begin() if self._capture_end is None else self._capture_end
      capture.set(core.StringPiece(self._capture_start, end - self._capture_start))
    return True

  def error(self):
    self._error = True
    return self

  @staticmethod
  def is_letter(ch):
    #return 97 <= ch <= 122 or 65 <= ch <= 90  # 'a' <= ch <= 'z' or 'A' <= ch <= 'Z'
    return Ascii.LOWER_A <= ch <= Ascii.LOWER_Z or Ascii.UPPER_A <= ch <= Ascii.UPPER_Z

  @staticmethod
  def is_lower_letter(ch):
    # return 97 <= ch <= 122  # 'a' <= ch <= 'z'
    return Ascii.LOWER_A <= ch <= Ascii.LOWER_Z

  @staticmethod
  def is_digit(ch):
    # return 48 <= ch <= 57  # '0' <= ch <= '9'
    return Ascii.DIGIT_0 <= ch <= Ascii.DIGIT_9

  @staticmethod
  def is_space(ch):
    return ch in [32, 9, 10, 11, 12, 13]  # list(map(ord, ' \t\n\v\f\r'))

  @staticmethod
  def matches(clz: CharClass, ch):
    if clz == Scanner.CharClass.ALL:
      return True
    elif clz == Scanner.CharClass.DIGIT:
      return Scanner.is_digit(ch)
    elif clz == Scanner.CharClass.LETTER:
      return Scanner.is_letter(ch)
    elif clz == Scanner.CharClass.LETTER_DIGIT:
      return Scanner.is_letter(ch) or Scanner.is_digit(ch)
    elif clz == Scanner.CharClass.LETTER_DIGIT_DASH_UNDERSCORE:
      return Scanner.is_letter(ch) or Scanner.is_digit(ch) \
             or ch == Ascii.DASH or ch == Ascii.UNDERSCORE
    elif clz == Scanner.CharClass.LETTER_DIGIT_DASH_DOT_SLASH:
      return Scanner.is_letter(ch) or Scanner.is_digit(ch) \
             or ch == Ascii.DASH or ch == Ascii.DOT or ch == Ascii.SLASH
    elif clz == Scanner.CharClass.LETTER_DIGIT_DASH_DOT_SLASH_UNDERSCORE:
      return Scanner.is_letter(ch) or Scanner.is_digit(ch) \
             or ch == Ascii.DASH or ch == Ascii.DOT or ch == Ascii.SLASH or ch == Ascii.UNDERSCORE
    elif clz == Scanner.CharClass.LETTER_DIGIT_DOT:
      return Scanner.is_letter(ch) or Scanner.is_digit(ch) \
             or ch == Ascii.DOT
    elif clz == Scanner.CharClass.LETTER_DIGIT_DOT_PLUS_MINUS:
      return Scanner.is_letter(ch) or Scanner.is_digit(ch) \
             or ch == Ascii.DOT or ch == Ascii.PLUS or ch == Ascii.MINUS
    elif clz == Scanner.CharClass.LETTER_DIGIT_DOT_UNDERSCORE:
      return Scanner.is_letter(ch) or Scanner.is_digit(ch) \
             or ch == Ascii.DOT or ch == Ascii.UNDERSCORE
    elif clz == Scanner.CharClass.LETTER_DIGIT_UNDERSCORE:
      return Scanner.is_letter(ch) or Scanner.is_digit(ch) \
             or ch == Ascii.UNDERSCORE
    elif clz == Scanner.CharClass.LOWERLETTER:
      return Scanner.is_lower_letter(ch)
    elif clz == Scanner.CharClass.LOWERLETTER_DIGIT:
      return Scanner.is_lower_letter(ch) or Scanner.is_digit(ch)
    elif clz == Scanner.CharClass.LOWERLETTER_DIGIT_UNDERSCORE:
      return Scanner.is_lower_letter(ch) or Scanner.is_digit(ch) \
             or ch == Ascii.UNDERSCORE
    elif clz == Scanner.CharClass.NON_ZERO_DIGIT:
      return Scanner.is_digit(ch) and ch != Ascii.DIGIT_0
    elif clz == Scanner.CharClass.SPACE:
      return Scanner.is_space(ch)
    elif clz == Scanner.CharClass.UPPERLETTER:
      return Ascii.UPPER_A <= ch <= Ascii.UPPER_Z
    elif clz == Scanner.CharClass.RANGLE:
      return ch == Ascii.RANGLE
    else:
      return False




'Generated protocol buffer code.'
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
_sym_db = _symbol_database.Default()
from ....tensorflow.core.framework import attr_value_pb2 as tensorflow_dot_core_dot_framework_dot_attr__value__pb2
from ....tensorflow.core.framework import node_def_pb2 as tensorflow_dot_core_dot_framework_dot_node__def__pb2
from ....tensorflow.core.framework import op_def_pb2 as tensorflow_dot_core_dot_framework_dot_op__def__pb2
DESCRIPTOR = _descriptor.FileDescriptor(name='tensorflow/core/framework/function.proto', package='tensorflow', syntax='proto3', serialized_options=b'\n\x18org.tensorflow.frameworkB\x0eFunctionProtosP\x01ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/function_go_proto\xf8\x01\x01', create_key=_descriptor._internal_create_key, serialized_pb=b'\n(tensorflow/core/framework/function.proto\x12\ntensorflow\x1a*tensorflow/core/framework/attr_value.proto\x1a(tensorflow/core/framework/node_def.proto\x1a&tensorflow/core/framework/op_def.proto"\xa8\x01\n\x12FunctionDefLibrary\x12)\n\x08function\x18\x01 \x03(\x0b2\x17.tensorflow.FunctionDef\x12)\n\x08gradient\x18\x02 \x03(\x0b2\x17.tensorflow.GradientDef\x12<\n\x14registered_gradients\x18\x03 \x03(\x0b2\x1e.tensorflow.RegisteredGradient"\xc4\x06\n\x0bFunctionDef\x12$\n\tsignature\x18\x01 \x01(\x0b2\x11.tensorflow.OpDef\x12/\n\x04attr\x18\x05 \x03(\x0b2!.tensorflow.FunctionDef.AttrEntry\x126\n\x08arg_attr\x18\x07 \x03(\x0b2$.tensorflow.FunctionDef.ArgAttrEntry\x12P\n\x16resource_arg_unique_id\x18\x08 \x03(\x0b20.tensorflow.FunctionDef.ResourceArgUniqueIdEntry\x12%\n\x08node_def\x18\x03 \x03(\x0b2\x13.tensorflow.NodeDef\x12-\n\x03ret\x18\x04 \x03(\x0b2 .tensorflow.FunctionDef.RetEntry\x12<\n\x0bcontrol_ret\x18\x06 \x03(\x0b2\'.tensorflow.FunctionDef.ControlRetEntry\x1aB\n\tAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b2\x15.tensorflow.AttrValue:\x028\x01\x1a\x88\x01\n\x08ArgAttrs\x128\n\x04attr\x18\x01 \x03(\x0b2*.tensorflow.FunctionDef.ArgAttrs.AttrEntry\x1aB\n\tAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b2\x15.tensorflow.AttrValue:\x028\x01\x1aP\n\x0cArgAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12/\n\x05value\x18\x02 \x01(\x0b2 .tensorflow.FunctionDef.ArgAttrs:\x028\x01\x1a:\n\x18ResourceArgUniqueIdEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12\r\n\x05value\x18\x02 \x01(\r:\x028\x01\x1a*\n\x08RetEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x028\x01\x1a1\n\x0fControlRetEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x028\x01J\x04\x08\x02\x10\x03";\n\x0bGradientDef\x12\x15\n\rfunction_name\x18\x01 \x01(\t\x12\x15\n\rgradient_func\x18\x02 \x01(\t"G\n\x12RegisteredGradient\x12\x15\n\rgradient_func\x18\x01 \x01(\t\x12\x1a\n\x12registered_op_type\x18\x02 \x01(\tB\x80\x01\n\x18org.tensorflow.frameworkB\x0eFunctionProtosP\x01ZOgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/function_go_proto\xf8\x01\x01b\x06proto3', dependencies=[tensorflow_dot_core_dot_framework_dot_attr__value__pb2.DESCRIPTOR, tensorflow_dot_core_dot_framework_dot_node__def__pb2.DESCRIPTOR, tensorflow_dot_core_dot_framework_dot_op__def__pb2.DESCRIPTOR])
_FUNCTIONDEFLIBRARY = _descriptor.Descriptor(name='FunctionDefLibrary', full_name='tensorflow.FunctionDefLibrary', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='function', full_name='tensorflow.FunctionDefLibrary.function', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='gradient', full_name='tensorflow.FunctionDefLibrary.gradient', index=1, number=2, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='registered_gradients', full_name='tensorflow.FunctionDefLibrary.registered_gradients', index=2, number=3, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=183, serialized_end=351)
_FUNCTIONDEF_ATTRENTRY = _descriptor.Descriptor(name='AttrEntry', full_name='tensorflow.FunctionDef.AttrEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.FunctionDef.AttrEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.FunctionDef.AttrEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=742, serialized_end=808)
_FUNCTIONDEF_ARGATTRS_ATTRENTRY = _descriptor.Descriptor(name='AttrEntry', full_name='tensorflow.FunctionDef.ArgAttrs.AttrEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.FunctionDef.ArgAttrs.AttrEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.FunctionDef.ArgAttrs.AttrEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=742, serialized_end=808)
_FUNCTIONDEF_ARGATTRS = _descriptor.Descriptor(name='ArgAttrs', full_name='tensorflow.FunctionDef.ArgAttrs', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='attr', full_name='tensorflow.FunctionDef.ArgAttrs.attr', index=0, number=1, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_FUNCTIONDEF_ARGATTRS_ATTRENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=811, serialized_end=947)
_FUNCTIONDEF_ARGATTRENTRY = _descriptor.Descriptor(name='ArgAttrEntry', full_name='tensorflow.FunctionDef.ArgAttrEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.FunctionDef.ArgAttrEntry.key', index=0, number=1, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.FunctionDef.ArgAttrEntry.value', index=1, number=2, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=949, serialized_end=1029)
_FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY = _descriptor.Descriptor(name='ResourceArgUniqueIdEntry', full_name='tensorflow.FunctionDef.ResourceArgUniqueIdEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.FunctionDef.ResourceArgUniqueIdEntry.key', index=0, number=1, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.FunctionDef.ResourceArgUniqueIdEntry.value', index=1, number=2, type=13, cpp_type=3, label=1, has_default_value=False, default_value=0, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1031, serialized_end=1089)
_FUNCTIONDEF_RETENTRY = _descriptor.Descriptor(name='RetEntry', full_name='tensorflow.FunctionDef.RetEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.FunctionDef.RetEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.FunctionDef.RetEntry.value', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1091, serialized_end=1133)
_FUNCTIONDEF_CONTROLRETENTRY = _descriptor.Descriptor(name='ControlRetEntry', full_name='tensorflow.FunctionDef.ControlRetEntry', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='key', full_name='tensorflow.FunctionDef.ControlRetEntry.key', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='value', full_name='tensorflow.FunctionDef.ControlRetEntry.value', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=b'8\x01', is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1135, serialized_end=1184)
_FUNCTIONDEF = _descriptor.Descriptor(name='FunctionDef', full_name='tensorflow.FunctionDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='signature', full_name='tensorflow.FunctionDef.signature', index=0, number=1, type=11, cpp_type=10, label=1, has_default_value=False, default_value=None, message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='attr', full_name='tensorflow.FunctionDef.attr', index=1, number=5, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='arg_attr', full_name='tensorflow.FunctionDef.arg_attr', index=2, number=7, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='resource_arg_unique_id', full_name='tensorflow.FunctionDef.resource_arg_unique_id', index=3, number=8, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='node_def', full_name='tensorflow.FunctionDef.node_def', index=4, number=3, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='ret', full_name='tensorflow.FunctionDef.ret', index=5, number=4, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='control_ret', full_name='tensorflow.FunctionDef.control_ret', index=6, number=6, type=11, cpp_type=10, label=3, has_default_value=False, default_value=[], message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[_FUNCTIONDEF_ATTRENTRY, _FUNCTIONDEF_ARGATTRS, _FUNCTIONDEF_ARGATTRENTRY, _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY, _FUNCTIONDEF_RETENTRY, _FUNCTIONDEF_CONTROLRETENTRY], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=354, serialized_end=1190)
_GRADIENTDEF = _descriptor.Descriptor(name='GradientDef', full_name='tensorflow.GradientDef', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='function_name', full_name='tensorflow.GradientDef.function_name', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='gradient_func', full_name='tensorflow.GradientDef.gradient_func', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1192, serialized_end=1251)
_REGISTEREDGRADIENT = _descriptor.Descriptor(name='RegisteredGradient', full_name='tensorflow.RegisteredGradient', filename=None, file=DESCRIPTOR, containing_type=None, create_key=_descriptor._internal_create_key, fields=[_descriptor.FieldDescriptor(name='gradient_func', full_name='tensorflow.RegisteredGradient.gradient_func', index=0, number=1, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key), _descriptor.FieldDescriptor(name='registered_op_type', full_name='tensorflow.RegisteredGradient.registered_op_type', index=1, number=2, type=9, cpp_type=9, label=1, has_default_value=False, default_value=b''.decode('utf-8'), message_type=None, enum_type=None, containing_type=None, is_extension=False, extension_scope=None, serialized_options=None, file=DESCRIPTOR, create_key=_descriptor._internal_create_key)], extensions=[], nested_types=[], enum_types=[], serialized_options=None, is_extendable=False, syntax='proto3', extension_ranges=[], oneofs=[], serialized_start=1253, serialized_end=1324)
_FUNCTIONDEFLIBRARY.fields_by_name['function'].message_type = _FUNCTIONDEF
_FUNCTIONDEFLIBRARY.fields_by_name['gradient'].message_type = _GRADIENTDEF
_FUNCTIONDEFLIBRARY.fields_by_name['registered_gradients'].message_type = _REGISTEREDGRADIENT
_FUNCTIONDEF_ATTRENTRY.fields_by_name['value'].message_type = tensorflow_dot_core_dot_framework_dot_attr__value__pb2._ATTRVALUE
_FUNCTIONDEF_ATTRENTRY.containing_type = _FUNCTIONDEF
_FUNCTIONDEF_ARGATTRS_ATTRENTRY.fields_by_name['value'].message_type = tensorflow_dot_core_dot_framework_dot_attr__value__pb2._ATTRVALUE
_FUNCTIONDEF_ARGATTRS_ATTRENTRY.containing_type = _FUNCTIONDEF_ARGATTRS
_FUNCTIONDEF_ARGATTRS.fields_by_name['attr'].message_type = _FUNCTIONDEF_ARGATTRS_ATTRENTRY
_FUNCTIONDEF_ARGATTRS.containing_type = _FUNCTIONDEF
_FUNCTIONDEF_ARGATTRENTRY.fields_by_name['value'].message_type = _FUNCTIONDEF_ARGATTRS
_FUNCTIONDEF_ARGATTRENTRY.containing_type = _FUNCTIONDEF
_FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY.containing_type = _FUNCTIONDEF
_FUNCTIONDEF_RETENTRY.containing_type = _FUNCTIONDEF
_FUNCTIONDEF_CONTROLRETENTRY.containing_type = _FUNCTIONDEF
_FUNCTIONDEF.fields_by_name['signature'].message_type = tensorflow_dot_core_dot_framework_dot_op__def__pb2._OPDEF
_FUNCTIONDEF.fields_by_name['attr'].message_type = _FUNCTIONDEF_ATTRENTRY
_FUNCTIONDEF.fields_by_name['arg_attr'].message_type = _FUNCTIONDEF_ARGATTRENTRY
_FUNCTIONDEF.fields_by_name['resource_arg_unique_id'].message_type = _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY
_FUNCTIONDEF.fields_by_name['node_def'].message_type = tensorflow_dot_core_dot_framework_dot_node__def__pb2._NODEDEF
_FUNCTIONDEF.fields_by_name['ret'].message_type = _FUNCTIONDEF_RETENTRY
_FUNCTIONDEF.fields_by_name['control_ret'].message_type = _FUNCTIONDEF_CONTROLRETENTRY
DESCRIPTOR.message_types_by_name['FunctionDefLibrary'] = _FUNCTIONDEFLIBRARY
DESCRIPTOR.message_types_by_name['FunctionDef'] = _FUNCTIONDEF
DESCRIPTOR.message_types_by_name['GradientDef'] = _GRADIENTDEF
DESCRIPTOR.message_types_by_name['RegisteredGradient'] = _REGISTEREDGRADIENT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)
FunctionDefLibrary = _reflection.GeneratedProtocolMessageType('FunctionDefLibrary', (_message.Message,), {'DESCRIPTOR': _FUNCTIONDEFLIBRARY, '__module__': 'tensorflow.core.framework.function_pb2'})
_sym_db.RegisterMessage(FunctionDefLibrary)
FunctionDef = _reflection.GeneratedProtocolMessageType('FunctionDef', (_message.Message,), {'AttrEntry': _reflection.GeneratedProtocolMessageType('AttrEntry', (_message.Message,), {'DESCRIPTOR': _FUNCTIONDEF_ATTRENTRY, '__module__': 'tensorflow.core.framework.function_pb2'}), 'ArgAttrs': _reflection.GeneratedProtocolMessageType('ArgAttrs', (_message.Message,), {'AttrEntry': _reflection.GeneratedProtocolMessageType('AttrEntry', (_message.Message,), {'DESCRIPTOR': _FUNCTIONDEF_ARGATTRS_ATTRENTRY, '__module__': 'tensorflow.core.framework.function_pb2'}), 'DESCRIPTOR': _FUNCTIONDEF_ARGATTRS, '__module__': 'tensorflow.core.framework.function_pb2'}), 'ArgAttrEntry': _reflection.GeneratedProtocolMessageType('ArgAttrEntry', (_message.Message,), {'DESCRIPTOR': _FUNCTIONDEF_ARGATTRENTRY, '__module__': 'tensorflow.core.framework.function_pb2'}), 'ResourceArgUniqueIdEntry': _reflection.GeneratedProtocolMessageType('ResourceArgUniqueIdEntry', (_message.Message,), {'DESCRIPTOR': _FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY, '__module__': 'tensorflow.core.framework.function_pb2'}), 'RetEntry': _reflection.GeneratedProtocolMessageType('RetEntry', (_message.Message,), {'DESCRIPTOR': _FUNCTIONDEF_RETENTRY, '__module__': 'tensorflow.core.framework.function_pb2'}), 'ControlRetEntry': _reflection.GeneratedProtocolMessageType('ControlRetEntry', (_message.Message,), {'DESCRIPTOR': _FUNCTIONDEF_CONTROLRETENTRY, '__module__': 'tensorflow.core.framework.function_pb2'}), 'DESCRIPTOR': _FUNCTIONDEF, '__module__': 'tensorflow.core.framework.function_pb2'})
_sym_db.RegisterMessage(FunctionDef)
_sym_db.RegisterMessage(FunctionDef.AttrEntry)
_sym_db.RegisterMessage(FunctionDef.ArgAttrs)
_sym_db.RegisterMessage(FunctionDef.ArgAttrs.AttrEntry)
_sym_db.RegisterMessage(FunctionDef.ArgAttrEntry)
_sym_db.RegisterMessage(FunctionDef.ResourceArgUniqueIdEntry)
_sym_db.RegisterMessage(FunctionDef.RetEntry)
_sym_db.RegisterMessage(FunctionDef.ControlRetEntry)
GradientDef = _reflection.GeneratedProtocolMessageType('GradientDef', (_message.Message,), {'DESCRIPTOR': _GRADIENTDEF, '__module__': 'tensorflow.core.framework.function_pb2'})
_sym_db.RegisterMessage(GradientDef)
RegisteredGradient = _reflection.GeneratedProtocolMessageType('RegisteredGradient', (_message.Message,), {'DESCRIPTOR': _REGISTEREDGRADIENT, '__module__': 'tensorflow.core.framework.function_pb2'})
_sym_db.RegisterMessage(RegisteredGradient)
DESCRIPTOR._options = None
_FUNCTIONDEF_ATTRENTRY._options = None
_FUNCTIONDEF_ARGATTRS_ATTRENTRY._options = None
_FUNCTIONDEF_ARGATTRENTRY._options = None
_FUNCTIONDEF_RESOURCEARGUNIQUEIDENTRY._options = None
_FUNCTIONDEF_RETENTRY._options = None
_FUNCTIONDEF_CONTROLRETENTRY._options = None



# /* Copyright 2016 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================*/
#
# #ifndef TENSORFLOW_CORE_LIB_IO_BUFFERED_INPUTSTREAM_H_
# #define TENSORFLOW_CORE_LIB_IO_BUFFERED_INPUTSTREAM_H_
#
# #include "tensorflow/core/lib/io/inputstream_interface.h"
# #include "tensorflow/core/platform/file_system.h"
#
# namespace tensorflow {
# namespace io {
#
# // Provides a buffer on top of an InputStreamInterface. A single instance of
# // BufferedInputStream is NOT safe for concurrent use by multiple threads.
# class BufferedInputStream : public InputStreamInterface {
#  public:
#   // Does not take ownership of input_stream unless owns_input_stream is set
#   // to true. input_stream must outlive *this then.
#   // TODO(rohanj): Remove owns_input_stream once the constructor below is
#   // removed.
#   BufferedInputStream(InputStreamInterface* input_stream, size_t buffer_bytes,
#                       bool owns_input_stream = false);
#
#   // For backwards compatibility, expose an interface that is similar to what
#   // InputBuffer exposes. Does not take ownership of file. file must outlive
#   // *this. This will be removed once we migrate all uses of this class to the
#   // constructor above.
#   BufferedInputStream(RandomAccessFile* file, size_t buffer_bytes);
#
#   ~BufferedInputStream() override;
#
#   tensorflow::Status ReadNBytes(int64 bytes_to_read, tstring* result) override;
#
#   tensorflow::Status SkipNBytes(int64 bytes_to_skip) override;
#
#   int64 Tell() const override;
#
#   // Seek to this offset within the file.
#   //
#   // If we seek to somewhere within our pre-buffered data, we will re-use what
#   // data we can.  Otherwise, Seek() throws out the current buffer and the next
#   // read will trigger an underlying read.
#   //
#   // Note: When seeking backwards in a stream, this implementation uses
#   // Reset() + SkipNBytes(), so its performance will be dependent
#   // largely on the performance of SkipNBytes().
#   tensorflow::Status Seek(int64 position);
#
#   // Read one text line of data into "*result" until end-of-file or a
#   // \n is read.  (The \n is not included in the result.)  Overwrites
#   // any existing data in *result.
#   //
#   // If successful, returns OK.  If we are already at the end of the
#   // file, we return an OUT_OF_RANGE error.  Otherwise, we return
#   // some other non-OK status.
#   tensorflow::Status ReadLine(std::string* result);
#   tensorflow::Status ReadLine(tstring* result);
#
#   // Returns one text line of data until end-of-file or a '\n' is read. The '\n'
#   // is included in the result.
#   // This method is a substitute for ReadLine() when called from Python which is
#   // the expectation in the python File::readline() API.
#   // Also, '\0's are treated like any other character within the line and given
#   // no special treatment.
#   std::string ReadLineAsString();
#
#   // Skip one text line of data.
#   //
#   // If successful, returns OK.  If we are already at the end of the
#   // file, we return an OUT_OF_RANGE error.  Otherwise, we return
#   // some other non-OK status.
#   tensorflow::Status SkipLine();
#
#   // Reads the entire contents of the file into *result.
#   //
#   // Note: the amount of memory used by this function call is unbounded, so only
#   // use in ops that expect that behavior.
#   template <typename T>
#   tensorflow::Status ReadAll(T* result);
#
#   tensorflow::Status Reset() override;
#
#  private:
#   tensorflow::Status FillBuffer();
#   template <typename StringType>
#   tensorflow::Status ReadLineHelper(StringType* result, bool include_eol);
#
#   InputStreamInterface* input_stream_;  // not owned.
#   size_t size_;                         // buffer size.
#   tstring buf_;                         // the buffer itself.
#   // buf_[pos_, limit_) holds the valid "read ahead" data in the file.
#   size_t pos_ = 0;    // current position in buf_.
#   size_t limit_ = 0;  // just past the end of valid data in buf_.
#   bool owns_input_stream_ = false;
#   // When EoF is reached, file_status_ contains the status to skip unnecessary
#   // buffer allocations.
#   tensorflow::Status file_status_ = Status::OK();
#
#   TF_DISALLOW_COPY_AND_ASSIGN(BufferedInputStream);
# };
#
# // Explicit instantiations defined in buffered_inputstream.cc.
# #ifndef SWIG
# extern template tensorflow::Status BufferedInputStream::ReadAll<std::string>(
#     std::string* result);
# extern template tensorflow::Status BufferedInputStream::ReadAll<tstring>(
#     tstring* result);
# #endif  // SWIG
#
# }  // namespace io
# }  // namespace tensorflow
#
# #endif  // TENSORFLOW_CORE_LIB_IO_BUFFERED_INPUTSTREAM_H_
from typing import Tuple, Optional

from tensorflow_checkpoint_reader import errors

# /* Copyright 2016 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================*/
#
# #include "tensorflow/core/lib/io/buffered_inputstream.h"
#
# #include "tensorflow/core/lib/io/random_inputstream.h"
#
# namespace tensorflow {
# namespace io {
#
# BufferedInputStream::BufferedInputStream(InputStreamInterface* input_stream,
#                                          size_t buffer_bytes,
#                                          bool owns_input_stream)
#     : input_stream_(input_stream),
#       size_(buffer_bytes),
#       owns_input_stream_(owns_input_stream) {
#   buf_.reserve(size_);
# }
#
# BufferedInputStream::BufferedInputStream(RandomAccessFile* file,
#                                          size_t buffer_bytes)
#     : BufferedInputStream(new RandomAccessInputStream(file), buffer_bytes,
#                           true) {}
#
# BufferedInputStream::~BufferedInputStream() {
#   if (owns_input_stream_) {
#     delete input_stream_;
#   }
# }
#
# Status BufferedInputStream::FillBuffer() {
#   if (!file_status_.ok()) {
#     pos_ = 0;
#     limit_ = 0;
#     return file_status_;
#   }
#   Status s = input_stream_->ReadNBytes(size_, &buf_);
#   pos_ = 0;
#   limit_ = buf_.size();
#   if (!s.ok()) {
#     file_status_ = s;
#   }
#   return s;
# }
#
# template <typename StringType>
# Status BufferedInputStream::ReadLineHelper(StringType* result,
#                                            bool include_eol) {
#   result->clear();
#   Status s;
#   size_t start_pos = pos_;
#   while (true) {
#     if (pos_ == limit_) {
#       result->append(buf_.data() + start_pos, pos_ - start_pos);
#       // Get more data into buffer
#       s = FillBuffer();
#       if (limit_ == 0) {
#         break;
#       }
#       start_pos = pos_;
#     }
#     char c = buf_[pos_];
#     if (c == '\n') {
#       result->append(buf_.data() + start_pos, pos_ - start_pos);
#       if (include_eol) {
#         result->append(1, c);
#       }
#       pos_++;
#       return Status::OK();
#     }
#     // We don't append '\r' to *result
#     if (c == '\r') {
#       result->append(buf_.data() + start_pos, pos_ - start_pos);
#       start_pos = pos_ + 1;
#     }
#     pos_++;
#   }
#   if (errors::IsOutOfRange(s) && !result->empty()) {
#     return Status::OK();
#   }
#   return s;
# }
#
# Status BufferedInputStream::ReadNBytes(int64 bytes_to_read, tstring* result) {
#   if (bytes_to_read < 0) {
#     return errors::InvalidArgument("Can't read a negative number of bytes: ",
#                                    bytes_to_read);
#   }
#   result->clear();
#   if (pos_ == limit_ && !file_status_.ok() && bytes_to_read > 0) {
#     return file_status_;
#   }
#   result->reserve(bytes_to_read);
#
#   Status s;
#   while (result->size() < static_cast<size_t>(bytes_to_read)) {
#     // Check whether the buffer is fully read or not.
#     if (pos_ == limit_) {
#       s = FillBuffer();
#       // If we didn't read any bytes, we're at the end of the file; break out.
#       if (limit_ == 0) {
#         DCHECK(!s.ok());
#         file_status_ = s;
#         break;
#       }
#     }
#     const int64 bytes_to_copy =
#         std::min<int64>(limit_ - pos_, bytes_to_read - result->size());
#     result->insert(result->size(), buf_, pos_, bytes_to_copy);
#     pos_ += bytes_to_copy;
#   }
#   // Filling the buffer might lead to a situation when we go past the end of
#   // the file leading to an OutOfRange() status return. But we might have
#   // obtained enough data to satisfy the function call. Returning OK then.
#   if (errors::IsOutOfRange(s) &&
#       (result->size() == static_cast<size_t>(bytes_to_read))) {
#     return Status::OK();
#   }
#   return s;
# }
#
# Status BufferedInputStream::SkipNBytes(int64 bytes_to_skip) {
#   if (bytes_to_skip < 0) {
#     return errors::InvalidArgument("Can only skip forward, not ",
#                                    bytes_to_skip);
#   }
#   if (pos_ + bytes_to_skip < limit_) {
#     // If we aren't skipping too much, then we can just move pos_;
#     pos_ += bytes_to_skip;
#   } else {
#     // Otherwise, we already have read limit_ - pos_, so skip the rest. At this
#     // point we need to get fresh data into the buffer, so reset pos_ and
#     // limit_.
#     Status s = input_stream_->SkipNBytes(bytes_to_skip - (limit_ - pos_));
#     pos_ = 0;
#     limit_ = 0;
#     if (errors::IsOutOfRange(s)) {
#       file_status_ = s;
#     }
#     return s;
#   }
#   return Status::OK();
# }
#
# int64 BufferedInputStream::Tell() const {
#   return input_stream_->Tell() - (limit_ - pos_);
# }
#
# Status BufferedInputStream::Seek(int64 position) {
#   if (position < 0) {
#     return errors::InvalidArgument("Seeking to a negative position: ",
#                                    position);
#   }
#
#   // Position of the buffer's lower limit within file.
#   const int64 buf_lower_limit = input_stream_->Tell() - limit_;
#   if (position < buf_lower_limit) {
#     // Seek before buffer, reset input stream and skip 'position' bytes.
#     TF_RETURN_IF_ERROR(Reset());
#     return SkipNBytes(position);
#   }
#
#   if (position < Tell()) {
#     // Seek within buffer before 'pos_'
#     pos_ -= Tell() - position;
#     return Status::OK();
#   }
#
#   // Seek after 'pos_'
#   return SkipNBytes(position - Tell());
# }
#
# template <typename T>
# Status BufferedInputStream::ReadAll(T* result) {
#   result->clear();
#   Status status;
#   while (status.ok()) {
#     status = FillBuffer();
#     if (limit_ == 0) {
#       break;
#     }
#     result->append(buf_);
#     pos_ = limit_;
#   }
#
#   if (errors::IsOutOfRange(status)) {
#     file_status_ = status;
#     return Status::OK();
#   }
#   return status;
# }
#
# template Status BufferedInputStream::ReadAll<std::string>(std::string* result);
# template Status BufferedInputStream::ReadAll<tstring>(tstring* result);
#
# Status BufferedInputStream::Reset() {
#   TF_RETURN_IF_ERROR(input_stream_->Reset());
#   pos_ = 0;
#   limit_ = 0;
#   file_status_ = Status::OK();
#   return Status::OK();
# }
#
# Status BufferedInputStream::ReadLine(std::string* result) {
#   return ReadLineHelper(result, false);
# }
#
# Status BufferedInputStream::ReadLine(tstring* result) {
#   return ReadLineHelper(result, false);
# }
#
# std::string BufferedInputStream::ReadLineAsString() {
#   std::string result;
#   ReadLineHelper(&result, true).IgnoreError();
#   return result;
# }
#
# Status BufferedInputStream::SkipLine() {
#   Status s;
#   bool skipped = false;
#   while (true) {
#     if (pos_ == limit_) {
#       // Get more data into buffer
#       s = FillBuffer();
#       if (limit_ == 0) {
#         break;
#       }
#     }
#     char c = buf_[pos_++];
#     skipped = true;
#     if (c == '\n') {
#       return Status::OK();
#     }
#   }
#   if (errors::IsOutOfRange(s) && skipped) {
#     return Status::OK();
#   }
#   return s;
# }
#
# }  // namespace io
# }  // namespace tensorflow

from . import inputstream_interface
from . import random_inputstream

class BufferedInputStream(inputstream_interface.InputStreamInterface):
  # BufferedInputStream::BufferedInputStream(InputStreamInterface* input_stream,
  #                                          size_t buffer_bytes,
  #                                          bool owns_input_stream)
  #     : input_stream_(input_stream),
  #       size_(buffer_bytes),
  #       owns_input_stream_(owns_input_stream) {
  #   buf_.reserve(size_);
  # }
  def __init__(self, input_stream: inputstream_interface.InputStreamInterface, buffer_bytes: int, owns_input_stream = False):
    assert isinstance(input_stream, inputstream_interface.InputStreamInterface)
    self.input_stream_ = input_stream
    self.size_ = buffer_bytes
    self.owns_input_stream_ = owns_input_stream

  # BufferedInputStream::BufferedInputStream(RandomAccessFile* file,
  #                                          size_t buffer_bytes)
  #     : BufferedInputStream(new RandomAccessInputStream(file), buffer_bytes,
  #                           true) {}

  # BufferedInputStream::~BufferedInputStream() {
  #   if (owns_input_stream_) {
  #     delete input_stream_;
  #   }
  # }

  def read_n_bytes(self, bytes_to_read: int) -> Tuple[errors.Status, Optional[bytearray]]:
    return self.input_stream_.read_n_bytes(bytes_to_read)

  def skip_n_bytes(self, bytes_to_skip: int) -> errors.Status:
    return self.input_stream_.skip_n_bytes(bytes_to_skip)

  # int64 BufferedInputStream::Tell() const {
  #   return input_stream_->Tell() - (limit_ - pos_);
  # }
  def tell(self) -> int:
    return self.input_stream_.tell()

  #   tensorflow::Status Seek(int64 position);
  def seek(self, position: int) -> errors.Status:
    """Seek to this offset within the file.

    If we seek to somewhere within our pre-buffered data, we will re-use what
    data we can.  Otherwise, Seek() throws out the current buffer and the next
    read will trigger an underlying read.

    Note: When seeking backwards in a stream, this implementation uses
    Reset() + SkipNBytes(), so its performance will be dependent
    largely on the performance of SkipNBytes().
    """
    raise NotImplementedError

  def reset(self):
    return self.input_stream_.reset()
